--[[ 
--------------------------------------
Main file for 3D Tic Tac Toe
By LMelior
Last updated: 20 August 2006
--------------------------------------
--]]

function init()
	
	System.usbDiskModeActivate()
	
	-- Set pseudo random number generator seed
	math.randomseed(os.time())
	-- Define global constants
	gameState = "menu" 		-- This is the current game state
	running = true			-- Tells the program when to end
	oldpad=Controls.read()	-- Initialize oldpad
	bsize=3					-- Initialize board size (default 3, can change to 4)
	sel=1
	AIdiff=1				-- Initialize AI difficulty (easy = 1, medium = 2, hard = 3)
	win=false
	draw=false
	
	-- Files to include
	dofile("menu.lua")
	dofile("AI.lua")
	
	-- Colors
	white = Color.new(255,255,255)
	red   = Color.new(255,000,000)
	green = Color.new(000,255,000)
	blue  = Color.new(000,000,255)
	black = Color.new(000,000,000)
	ch2color={red,white}
	
	-- Images
	prescreen=Image.createEmpty(480,272)
	background=Image.load("./bgs/title.png")
	selector1=Image.load("./four/blue.png")
	selector2=Image.load("./four/red.png")
	buttons=Image.load("buttons.png")
	
	-- Sounds
	
	
end


function gameInit()
	
	initpos={x=2,y=2,z=2}	-- initial tile position
	curtpos={x=2,y=2,z=2}	-- current tile position
	turn=math.random(2)
	win=false
	draw=false
	gameState="game"
	
	if bsize==3 then
		dofile("./three/wincombo.lua")
		cursor=Image.load("./three/cursor.png")
		tile=Image.load("./three/block.png")
		p1=Image.load("./three/red.png")
		p2=Image.load("./three/blue.png")
		cenppos={x=215,y=111}	-- center tile pixel position
		-- Create blank bsize x bsize x bsize map
		mapmat = {}
		for i=1,bsize do
			mapmat[i] = {}     	
			for j=1,bsize do
				mapmat[i][j] = {}
				for k=1,bsize do
					mapmat[i][j][k] = 0
				end
			end
		end
	elseif bsize==4 then
		dofile("./four/wincombo4.lua")
		cursor=Image.load("./four/cursor.png")
		tile=Image.load("./four/block.png")
		p1=Image.load("./four/red.png")
		p2=Image.load("./four/blue.png")
		cenppos={x=215,y=111}	-- center tile pixel position
		-- Create blank bsize x bsize x bsize map
		mapmat = {}
		for i=1,bsize do
			mapmat[i] = {}     	
			for j=1,bsize do
				mapmat[i][j] = {}
				for k=1,bsize do
					mapmat[i][j][k] = 0
				end
			end
		end
	end
	
end

function playerMove()
		
	-- Allow player to change cursor position
	if pad:up() and oldpad:up()~=pad:up() and curtpos.y>1 then
		curtpos.y=curtpos.y-1
	elseif pad:down() and oldpad:down()~=pad:down() and curtpos.y<bsize then
		curtpos.y=curtpos.y+1
	elseif pad:left() and oldpad:left()~=pad:left() and curtpos.x<bsize then
		curtpos.x=curtpos.x+1
	elseif pad:right() and oldpad:right()~=pad:right() and curtpos.x>1 then
		curtpos.x=curtpos.x-1
	elseif pad:triangle() and oldpad:triangle()~=pad:triangle() and curtpos.z>1 then
		curtpos.z=curtpos.z-1
	elseif pad:cross() and oldpad:cross()~=pad:cross() and curtpos.z<bsize then
		curtpos.z=curtpos.z+1
	-- place piece if spot is open
	elseif pad:circle() and oldpad:circle()~=pad:circle() and mapmat[curtpos.y][curtpos.x][curtpos.z]==0 then
		mapmat[curtpos.y][curtpos.x][curtpos.z]=1
		turn=2
	elseif pad:start() then
		running=false
	end

	
end


function blitall()

	-- Blit background
	screen:blit(0,0,background)
	
	-- Blit map to prescreen, rear tiles first
	if bsize==3 then
		for k=3,1,-1 do
			for i=1,3 do
				for j=1,3 do
					--local xloc=cenppos.x+(initpos.x-j)*16-(initpos.y-i)*16+mapoffset.x
					--local yloc=cenppos.y-(initpos.x-j)*8-(initpos.y-i)*8+mapoffset.y
					local xloc=215+30*(2-k)+(2-j)*24-(2-i)*24
					local yloc=86*k-60-(2-j)*12-(2-i)*12
					if mapmat[i][j][k]==0 then
						prescreen:blit(xloc,yloc,tile)
					elseif mapmat[i][j][k]==1 then
						prescreen:blit(xloc,yloc,p1)
					elseif mapmat[i][j][k]==2 then
						prescreen:blit(xloc,yloc,p2)
					end
					if i==curtpos.y and j==curtpos.x and k==curtpos.z then
						prescreen:blit(xloc,yloc,cursor)
					end
				end
			end
		end
		-- Blit tile cursor to map
		--prescreen:blit(cenppos.x+30*(2-curtpos.z)+(initpos.x-curtpos.x)*24-(initpos.y-curtpos.y)*24,86*curtpos.z-60-(initpos.x-curtpos.x)*12-(initpos.y-curtpos.y)*12,cursor)

	elseif bsize==4 then
		for k=4,1,-1 do
			for i=1,4 do
				for j=1,4 do
					--local xloc=cenppos.x+(initpos.x-j)*16-(initpos.y-i)*16+mapoffset.x
					--local yloc=cenppos.y-(initpos.x-j)*8-(initpos.y-i)*8+mapoffset.y
					local xloc=200+40*(2-k)+(2-j)*18-(2-i)*18
					local yloc=65*k-53-(2-j)*9-(2-i)*9
					if mapmat[i][j][k]==0 then
						prescreen:blit(xloc,yloc,tile)
					elseif mapmat[i][j][k]==1 then
						prescreen:blit(xloc,yloc,p1)
					elseif mapmat[i][j][k]==2 then
						prescreen:blit(xloc,yloc,p2)
					end
					if i==curtpos.y and j==curtpos.x and k==curtpos.z then
						prescreen:blit(xloc,yloc,cursor)
					end
				end
			end
		end
		-- Blit tile cursor to map
		--prescreen:blit(cenppos.x+30*(2-curtpos.z)+(initpos.x-curtpos.x)*18-(initpos.y-curtpos.y)*18,75*curtpos.z-60-(initpos.x-curtpos.x)*9-(initpos.y-curtpos.y)*9,cursor)

	end
		
	-- Blit prescreen to screen
	screen:blit(0,0,prescreen)
	
	-- print instructions
	screen:blit(1,1,buttons,15,0,15,15)
	screen:print(20,5,"Claim square",white)
	screen:blit(341,236,buttons,45,0,15,15)
	screen:print(360,240,"Move up level",white)
	screen:blit(341,256,buttons,0,0,15,15)
	screen:print(360,260,"Move down level",white)
	

end


function checkWin(player)
	if bsize==3 then
		for k=1,49 do
			if mapmat[wincombo[k].y1][wincombo[k].x1][wincombo[k].z1]==player and mapmat[wincombo[k].y2][wincombo[k].x2][wincombo[k].z2]==player and mapmat[wincombo[k].y3][wincombo[k].x3][wincombo[k].z3]==player then
				win=true
				background=Image.load("./bgs/"..player.."pwins.png")
				while true do
					screen:clear()
					screen:blit(0,0,background)
					pad=Controls.read()
					if pad:cross() then 
						gameState="menu"
						background=Image.load("./bgs/title.png")
						break
					end
					screen.waitVblankStart()
					screen:flip()
				end
			end
		end
	elseif bsize==4 then
		for k=1,76 do
			if mapmat[wincombo[k].y1][wincombo[k].x1][wincombo[k].z1]==player and mapmat[wincombo[k].y2][wincombo[k].x2][wincombo[k].z2]==player and mapmat[wincombo[k].y3][wincombo[k].x3][wincombo[k].z3]==player and mapmat[wincombo[k].y4][wincombo[k].x4][wincombo[k].z4]==player then
				win=true
				background=Image.load("./bgs/"..player.."pwins.png")
				while true do
					screen:clear()
					screen:blit(0,0,background)
					pad=Controls.read()
					if pad:cross() then 
						gameState="menu"
						background=Image.load("./bgs/title.png")
						break
					end
					screen.waitVblankStart()
					screen:flip()
				end
			end
		end
	end
end


function checkDraw()
	local empty=0
	for i=1,bsize do 	
		for j=1,bsize do
			for k=1,bsize do
				if mapmat[i][j][k] == 0 then empty=empty+1 end
			end
		end
	end
	if empty == 0 then
	draw=true
		background=Image.load("./bgs/draw.png")
		while true do
			screen:clear()
			screen:blit(0,0,background)
			pad=Controls.read()
			if pad:cross() then 
				gameState="menu"
				background=Image.load("./bgs/title.png")
				break
			end
			screen.waitVblankStart()
			screen:flip()
		end
	end
end


function Game()
	
	if turn==1 and not win and not draw then playerMove() end
	checkWin(1)
	checkDraw()
	if gameState=="game" then blitall() end
	if turn==2 and not win and not draw then AIMove() end
	checkWin(2)
	checkDraw()
	if gameState=="game" then blitall() end
	
	if pad:start() then
		running=false
	end
	
end


function shutdown()

	-- Unload things that take up a lot of memory (images, sounds)
	prescreen=nil
	cursor=nil
	tile=nil
	p1=nil
	p2=nil
	wincombo=nil
	
end


function main()

init()

	while running do
		prescreen:clear()
		screen:clear()
		pad=Controls.read()

		if gameState == "menu" then
			MainMenu()
		elseif gameState == "gameInit" then
			gameInit()
		elseif gameState == "game" then
			Game()
		else
			break
		end
		
		screen.waitVblankStart()
		screen.flip()
		oldpad=pad
	end

	shutdown()
	
end


main()
